#!/bin/bash

##########################################################################################
## 06/26/2018 Justin Rajendra
## a script to install and configure afni on macOS
## bash version with apple script beta 2.0

## just get a clean line
echo ""

## current versions of programs
## newer R...
# R_ver="3.5.0"
# R_link="https://cran.r-project.org/bin/macosx/R-${R_ver}.pkg"
# R_clang_ver="6.0.0"
# R_gfortran_ver="6.1"

## older R for Rmpi
R_ver="3.4.1"
R_link="https://cloud.r-project.org/bin/macosx/el-capitan/base/R-${R_ver}.pkg"

## xquartz
XQ_ver="2.7.11"

## check macOS version and select the correct afni build
mac_ver_full=`defaults read loginwindow SystemVersionStampAsString`
IFS=’.’ read -ra mac_ver <<< "$mac_ver_full"
if [ "${mac_ver[1]}" -lt "7" ]; then
    echo ""
    echo "** ERROR: You must have at least OSX version 10.7!!"
    echo ""
elif [ "${mac_ver[1]}" -lt "12" ]; then
    afni_ver="macosx_10.7_local"
else
    afni_ver="macos_10.12_local"
fi
afni_web="https://afni.nimh.nih.gov/pub/dist/bin"
afni_link="${afni_web}/${afni_ver}/@update.afni.binaries"

## brew stuff
brew_cmd='/usr/bin/ruby -e "$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/master/install)"'
brew_pkgs="netpbm openmpi"
IFS=' ' read -r -a brew_pkgs_arr <<< "$brew_pkgs"   ## make into array

#####################################################################
## let's try functions for apple gui input

## Get sudo password from text box
## takes text and title for the question and outputs the string
function AppleSUDO () {
/usr/bin/osascript <<EOT
	tell application "System Events"
		activate
        with timeout of 36000 seconds
    		set subj to text returned of (display dialog "$text"¬
    		default answer "" with icon 2 with title "${title}" with hidden answer)
        end
	end tell
EOT
}	## end function sudo text input

## Get user input from text box
## takes text and title for the question and outputs the string
function AppleTextInput () {
/usr/bin/osascript <<EOT
	tell application "System Events"
		activate
        with timeout of 36000 seconds
    		set subj to text returned of (display dialog "${text}"¬
    		default answer "" with icon 2 with title "${title}")
        end
	end tell
EOT
}	## end function text input

## two button choices
## input text, title, choice1, choice2 outputs choice as string
function AppleChoose2 () {
/usr/bin/osascript <<EOT
	tell application "System Events"
        activate
        with timeout of 36000 seconds
    		set response to display dialog "${text}"  ¬
    		buttons {"${choice1}", "${choice2}"}  ¬
            default button "${choice1}" with title "${title}" with icon 2
        end
    end tell
    if response={button returned:"${choice1}"} then
        set answer to "${choice1}"
    else if response={button returned:"${choice2}"} then
        set answer to "${choice2}"
    end if
EOT
}	## end function choose 2

## select path to folder
## input text, folder, outputs path as string
function AppleFolder () {
/usr/bin/osascript <<EOT
	tell application "Finder"
		activate
        with timeout of 36000 seconds
    		set PathToFolder to (choose folder with prompt "${text}" ¬
    		default location "$folder") as string
    		set PathToFolder2 to POSIX path of PathToFolder
        end
	end tell
EOT
}	## end function choose folder

## select path to file
## input text, folder, outputs path/file as string
function AppleFile () {
/usr/bin/osascript <<EOT
	tell application "Finder"
    	activate
        with timeout of 36000 seconds
        	set PathToFile to (choose file with prompt "${text}" ¬
        	default location "${folder}") as string
        	set PathToFile2 to POSIX path of PathToFile
        end
	end tell
EOT
}	## end function choose file

## info dialog
## input text, title, button, outputs nothing
## icon 0=error, 1=application, 2=warning
function AppleInfo () {
/usr/bin/osascript <<EOT
	tell application "System Events"
		activate
        with timeout of 36000 seconds
    		set response to display dialog "${text}"  ¬
    		buttons {"${button}"} default button "${button}"  ¬
            with title "${title}" with icon ${icon}
        end
	end tell
EOT
}	## end function info dialog

## multiple selection
## input nothing, outputs selections
function AppleChooseMultiple () {
/usr/bin/osascript <<EOT
	tell application "System Events"
		activate
        with timeout of 36000 seconds
            choose from list {"Xcode Command Line Tools","XQuartz",¬
            "R (${R_ver})","AFNI","Class data download",¬
            "afni_system_check","Shiny packages",¬
            "Homebrew (for ${brew_pkgs})"} ¬
            with title "What should we install?" ¬
            with prompt "Choose one or more (⌘+click):" ¬
            OK button name "Install" ¬
            cancel button name "Quit" ¬
            default items ¬
            {"Xcode Command Line Tools","XQuartz","R (${R_ver})","AFNI","afni_system_check"} ¬
            with multiple selections allowed
        end
	end tell
EOT
}

###########################################################################
## temp folder for installation
inst_dir="$HOME/afni_install_temp_delete_me"
if [ -d "${inst_dir}" ]; then
    \rm -r $inst_dir
fi
\mkdir -p $inst_dir
cd $inst_dir

###########################################################################
## clean up on and for ctrl-c
function CleanGood () {
    echo "" ; echo "Cleaning up!"
    if [ -d "/Volumes/XQuartz-${XQ_ver}" ]; then
        hdiutil unmount -force "/Volumes/XQuartz-${XQ_ver}"
    fi
    \rm -rfv ${inst_dir}
    echo "" ; echo "Thank you and goodbye." ; echo "" ; exit 0
}
function CleanBad () {
    echo "" ; echo "Cleaning up!"
    if [ -d "/Volumes/XQuartz-${XQ_ver}" ]; then
        hdiutil unmount -force "/Volumes/XQuartz-${XQ_ver}"
    fi
    \rm -rfv ${inst_dir}
    echo "" ; echo "Something bad happened. Try again..." ; echo "" ; exit 1
}
trap 'CleanBad' INT

## check for afni
function has_afni () {
    afni_bin=`which afni`
    if [ "$afni_bin" = "" ]; then
        text1="AFNI is not found, but is needed to continue the installation!!\n\n"
        text2="Please check your AFNI install and try again!"
        text=`echo ${text1}${text2}`
        title="AFNI is not found!!" ; button="Quit!" ; icon=0
        junk=`AppleInfo ${text} ${title} ${button} ${icon}`
        CleanBad
    fi
}

###########################################################################
## welcome
text1="This script will install most of what you need to run afni and related"
text2=" programs.\n\nYou will need to enter in your administrator password"
text3=" for some packages.\n\n"
text4="The next window will give you choices of what to install. "
text5="The preselected choices are recommended, "
text6="but you can find more information here:\n\n"
text7="https://afni.nimh.nih.gov/pub/dist/doc/htmldoc/background_install/install_instructs/steps_mac_app.html"
text=`echo ${text1}${text2}${text3}${text4}${text5}${text6}${text7}`
title="Welcome to the afni installation for  macOS !!"
choice1="Let's go!" ; choice2="Quit"
LetsGo=`AppleChoose2`
if [[ "${LetsGo}" = "Quit" ]]; then
    CleanGood
fi

###########################################################################
## choose what to install and make array
inst_choices=`AppleChooseMultiple`
if [[ "${inst_choices[@]}" =~ "false" ]]; then
    echo "" ; echo "Thank you and goodbye!!" ; echo "" ; exit 0
else
    IFS=', ' read -r -a inst_list <<< "$inst_choices"
fi

###########################################################################
## get sudo password if needed
# if [[ "${inst_list[@]}" =~ "Xcode" ]] || [[ "${inst_list[@]}" =~ "XQuartz" ]] \
#     || [[ "${inst_list[@]}" =~ "R" ]] || [[ "${inst_list[@]}" =~ "AFNI" ]]; then
if [[ "${inst_list[@]}" =~ "Xcode" ]] || [[ "${inst_list[@]}" =~ "XQuartz" ]] \
    || [[ "${inst_list[@]}" =~ "R" ]]; then
    text="Please enter your password to be used for any 'sudo' commands:"
    title="Do you have the power?"
    mast_pass=`AppleSUDO`

    ## check sudo
    echo "${mast_pass}" | sudo -kS pwd 2> /dev/null
    if [[ $? != 0 ]]; then
        text1="Your password is invalid or you do not have 'sudo' powers!\n"
        text2="Double check your password and try again."
        text=`echo ${text1}${text2}`
        title="No power!" ; button="Quit!" ; icon=0 ; junk=`AppleInfo`
        CleanBad
    fi
else
    mast_pass=""
fi

###########################################################################
## xcode command line tools
if [[ "${inst_list[@]}" =~ "Xcode" ]]; then
# rm -rf /Library/Developer/CommandLineTools
    ## try the headless install
    if [ ! -f "/Library/Developer/CommandLineTools/usr/bin/git" ]; then

        ## make a temp file to trick softwareupdate into thinking it needs it
        echo ""
        echo "******************************************************"
        echo "Preparing Xcode Command Line Tools" ; echo ""
        XC_tmp="/tmp/.com.apple.dt.CommandLineTools.installondemand.in-progress"
        echo "${mast_pass}" | sudo -kS touch $XC_tmp 2> /dev/null

        ## get the label
        XC_item=`softwareupdate -l | grep -B 1 -E "Command Line (Developer|Tools)" | awk -F"*" '/^ +\\*/ {print $2}' | sed 's/^ *//' | tail -n1`

        ## install from softwareupdate and clean up
        echo "" ; echo "Installing ${XC_item}" ; echo ""
        echo ${mast_pass} | sudo -kS softwareupdate -i "${XC_item}" 2> /dev/null
        echo ${mast_pass} | sudo -kS rm -f $XC_tmp 2> /dev/null
        echo ${mast_pass} | sudo -kS xcode-select --switch /Library/Developer/CommandLineTools 2> /dev/null

        ## try the GUI install
        if [ ! -f "/Library/Developer/CommandLineTools/usr/bin/git" ]; then

            text1="The Xcode Command Line Tools installation GUI will pop up next."
            text2="After you complete the installation, go back to the terminal"
            text3="and press enter." ; text=`echo ${text1} ${text2} ${text3}`
            title="Xcode Command Line Tools second try..." ; button="OK"
            icon=1 ; junk=`AppleInfo`

            ## start the gui and wait at the terminal
            echo "" ; xcode-select --install ; echo ""
            echo "Press 'enter' **AFTER** the Xcode Command Line Tools installation is complete."
            read temp

            ## failed all tries
            if [ ! -f "/Library/Developer/CommandLineTools/usr/bin/git" ]; then
                text1="Xcode Command Line Tools installation failed!"
                text2="Try typing 'xcode-select --install' into the command"
                text3="line, and running the AFNI installation again."
                text=`echo ${text1} ${text2} ${text3}`
                title="FAILED!" ; button="Quit!" ; icon=0
                junk=`AppleInfo` ; CleanBad
            else
                echo ${mast_pass} | sudo -kS \
                xcode-select --switch /Library/Developer/CommandLineTools \
                2> /dev/null
            fi
        fi
    ## already there?
    else
        text="Xcode Command Line Tools are already installed!"
        title="Skipping" ; button="Skip" ; icon=2 ; junk=`AppleInfo`
    fi

fi   ## end xcode command line tools

###########################################################################
## Xquartz
if [[ "${inst_list[@]}" =~ "XQuartz" ]]; then
    XQ_inst="no"
    XQ_bin=`which XQuartz`

    ## found, reinstall?
    if [ "$XQ_bin" != "" ]; then
        text="Reinstall XQuartz?" ; title="Xquartz is installed at ${XQ_bin}"
        choice1="Reinstall" ; choice2="Skip"
        XQ_reinstall=`AppleChoose2`
        if [ $XQ_reinstall = "Reinstall" ]; then
            XQ_inst="yes"
        fi
    else
        XQ_inst="yes"
    fi   ## end exists check

    ## download and install
    if [ $XQ_inst = "yes" ]; then
        ## if already mounted, force eject
        if [ -d "/Volumes/XQuartz-${XQ_ver}" ]; then
            hdiutil unmount -force "/Volumes/XQuartz-${XQ_ver}"
        fi

        ## download with redirects and make sure it finishes
        echo ""
        echo "******************************************************"
        echo "Downloading XQuartz-${XQ_ver}" ; echo ""
        curl -LO "https://dl.bintray.com/xquartz/downloads/XQuartz-${XQ_ver}.dmg"
        if [ -f "XQuartz-${XQ_ver}.dmg" ]; then

            ## mount it
            hdiutil attach -quiet XQuartz-${XQ_ver}.dmg

            # echo ""; echo "Enter your 'sudo' password to install XQuartz:"; echo ""
            echo ${mast_pass} | sudo -kS \
            installer -verbose -pkg /Volumes/XQuartz-${XQ_ver}/XQuartz.pkg -target / \
            2> /dev/null

            ## check to see if it failed
            if [[ $? != 0 ]]; then
                text1="XQuartz installation failed. Do you have 'sudo' powers?"
                text2="Please install XQuartz manually (https://www.xquartz.org/)"
                text3="and rerun this program."
                text=`echo ${text1} ${text2} ${text3}`
                title="XQuartz installation failed!" ; button="Quit!" ; icon=0
                junk=`AppleInfo` ; CleanBad
            fi

            ## clean up a little
            hdiutil unmount "/Volumes/XQuartz-${XQ_ver}"
            \rm XQuartz-${XQ_ver}.dmg

        else
            text1="Please install XQuartz manually (https://www.xquartz.org/)"
            text2="and rerun this program."
            text=`echo ${text1} ${text2}`
            title="XQuartz download failed!" ; button="Quit!" ; icon=0
            junk=`AppleInfo` ; CleanBad
        fi
    fi   ## end XQ download and install

    ## click through inactive windows
    defaults write org.macosforge.xquartz.X11 wm_click_through -bool true
fi   ## end Xquartz

###########################################################################
## R installation
if [[ "${inst_list[@]}" =~ "R" ]]; then

   #  sudo rm -Rf /Library/Frameworks/R.framework /Applications/R.app \
   # /usr/local/bin/R /usr/local/bin/Rscript
    ## check it
    R_inst="no"
    R_bin=`which R`
    if [ "$R_bin" != "" ]; then
        ## get current version of installed R
        R_cur_ver=`R --version | grep "R version" | cut -d " " -f 3`

        ## ask for upgrade/reinstall or skip
        text1="R version ${R_cur_ver} is installed."
        text2="Would you like to install R version ${R_ver}?"
        text=`echo ${text1} ${text2}` ; title="R is installed at ${R_bin}"
        choice1="Reinstall" ; choice2="Skip"
        R_reinstall=`AppleChoose2`
        if [ $R_reinstall = "Reinstall" ]; then
            R_inst="yes"
        fi
    else
        R_inst="yes"
    fi   ## end exists check

    ## download and install
    if [ $R_inst = "yes" ]; then

        ## download and check and install
        echo ""
        echo "******************************************************"
        echo "Downloading R (${R_ver})" ; echo ""
        curl -LO "${R_link}"
        if [ -f "R-${R_ver}.pkg" ]; then

            echo ""; echo "Installing R (${R_ver})"
            echo ${mast_pass} | sudo -kS \
            installer -verbose -pkg R-${R_ver}.pkg -target / \
            2> /dev/null

            ## check to see if it failed
            if [[ $? != 0 ]]; then
                text1="R installation failed. Do you have 'sudo' powers?"
                text2="Please install R manually (${R_link})"
                text3="and rerun this program."
                text=`echo ${text1} ${text2} ${text3}`
                title="R installation failed!" ; button="Quit!" ; icon=0
                junk=`AppleInfo` ; CleanBad
            fi
            echo "" ; echo "R (${R_ver}) install complete."
        else
            text1="Please install R manually (${R_link})"
            text2="and rerun this program."
            text=`echo ${text1} ${text2}`
            title="R download failed!" ; button="Quit!" ; icon=0
            junk=`AppleInfo` ; CleanBad
        fi
    fi   ## end install
fi   ## end R check

###########################################################################
## AFNI
if [[ "${inst_list[@]}" =~ "AFNI" ]]; then

    ## check for R first
    R_bin=`which R`
    if [ "$R_bin" = "" ]; then
        text1="R seems to be missing!\n"
        text2="Please check your R installation or download from here:\n\n"
        text3="${R_link}"
        text=`echo ${text1}${text2}${text3}`
        title="R is missing!!" ; button="Quit!" ; icon=0 ; junk=`AppleInfo`
        CleanBad
    fi   ## end R check

    ## check if it is there
    afni_inst="no"
    afni_bin=`which afni`
    if [ "$afni_bin" != "" ]; then
        text="Do you want to reinstall AFNI?"
        title="AFNI is already installed!"
        choice1="Reinstall" ; choice2="Skip"
        afni_reinstall=`AppleChoose2`
        if [ $afni_reinstall == "Reinstall" ]; then
            afni_inst="yes"
        fi
    else
        afni_inst="yes"
    fi   ## end check if afni is there

    ############################################
    ## install afni
    if [ "$afni_inst" = "yes" ]; then

        ## get the install script either way
        echo ""
        echo "******************************************************"
        echo "Downloading @update.afni.binaries" ; echo ""
        curl -LO "${afni_link}"

        ## check the download
        if [ ! -f "@update.afni.binaries" ]; then
            text1="AFNI download failed!\n"
            text2="Please follow instructions here: \n\n"
            text3="https://afni.nimh.nih.gov/pub/dist/doc/htmldoc/background_install/install_instructs/steps_mac.html"
            text=`echo ${text1}${text2}${text3}`
            title="AFNI download failed!" ; button="Quit!" ; icon=0
            junk=`AppleInfo` ; CleanBad
        fi

        ## ask for default folder
        text1="Do you want to install AFNI in the default location"
        text2="(~/abin) or somewhere else?"
        text=`echo ${text1} ${text2}`
        title="Where should we put AFNI?"
        choice1="~/abin" ; choice2="Elsewhere"
        afni_dest=`AppleChoose2`

        ## get target folder and make sure they don't cancel
        if [ "$afni_dest" = "Elsewhere" ]; then
            text="Select or create target a folder for AFNI installation."
            folder="$HOME"
            bindir=`AppleFolder`
            if [ "$bindir" = "" ]; then
                text1="You cancelled folder selection!\n"
                text2="Please try again if you make up your mind."
                text=`echo ${text1}${text2}`
                title="Folder selection cancelled!" ; button="Quit!" ; icon=0
                junk=`AppleInfo` ; CleanBad
            fi
        else
            bindir="$HOME/abin"
        fi

        ## warn if stuff is there
        if [ ! -z "$(ls -A $bindir)" ]; then
            text1="$bindir is not empty!\n"
            text2="Do you want to install AFNI there anyway?\n"
            text3="The current AFNI install will be backed up.\n"
            text=`echo ${text1}${text2}${text3}`
            title="Stuff is there!" ; choice1="Install" ; choice2="Skip"
            afni_overwrite=`AppleChoose2`
            if [ $afni_overwrite == "Install" ]; then
                tcsh @update.afni.binaries -defaults -bindir ${bindir} \
                                           -package ${afni_ver} -do_dotfiles
            fi
        else
            tcsh @update.afni.binaries -defaults -bindir ${bindir} \
                                       -package ${afni_ver}
        fi  ## end target folder check

        ## check to see if it failed
        if [[ $? != 0 ]]; then
            text1="@update.afni.binaries failed!\n"
            text2="Do you have write permissions for ${bindir}?"
            text=`echo ${text1}${text2}`
            title="AFNI installation failed!"
            button="Quit!" ; icon=0 ; junk=`AppleInfo` ; CleanBad
        fi
    fi   ## end afni install

    ############################################
    ## install R packages

    ## need to add afni to the path and make sure it is there
    export PATH="$PATH:${bindir}"
    has_afni

    echo ""
    echo "******************************************************"
    echo "Running rPkgsInstall" ; echo "This may take awhile..." ; echo ""
    # echo ${mast_pass} | sudo -kS rPkgsInstall -pkgs ALL 2> /dev/null
    rPkgsInstall -pkgs ALL

    ## check to see if it failed
    if [[ $? != 0 ]]; then
        text1="rPkgsInstall failed. Do you have 'sudo' powers?\n"
        text2="If you don't, you can try:\n\nrPkgsInstall -pkgs ALL\n"
        text3="Or you can get your admin to run:\n\nsudo rPkgsInstall -pkgs ALL"
        text=`echo ${text1}${text2}${text3}`
        title="rPkgsInstall installation failed!" ; button="Quit!" ; icon=0
        junk=`AppleInfo` ; CleanBad
    fi
    echo "" ; echo "Done with rPkgsInstall" ; echo ""

    ############################################
    ## extras for paths fix this stuff please
    csh_add='setenv DYLD_LIBRARY_PATH /opt/X11/lib/flat_namespace'
    bash_add='export DYLD_LIBRARY_PATH=$DYLD_LIBRARY_PATH:/opt/X11/lib/flat_namespace'

    echo ""
    echo "******************************************************"
    echo "Updating .cshrc, .bashrc, .bash_profile, and environment" ; echo ""
    touch ~/.cshrc
    temp_grep=`grep -c "/opt/X11/lib/flat_namespace" ~/.cshrc`
    if [ "$temp_grep" = "0" ]; then
        echo ${csh_add} >> ~/.cshrc
        echo "Updated .cshrc."
    fi
    touch ~/.bashrc
    temp_grep=`grep -c "/opt/X11/lib/flat_namespace" ~/.bashrc`
    if [ "$temp_grep" = "0" ]; then
        echo ${bash_add} >> ~/.bashrc
        echo 'ahdir=`apsearch -afni_help_dir`' >> ~/.bashrc
        echo 'if [ -f "$ahdir/all_progs.COMP.bash" ]' >> ~/.bashrc
        echo "then" >> ~/.bashrc
        echo '   . $ahdir/all_progs.COMP.bash' >> ~/.bashrc
        echo "fi" >> ~/.bashrc
        echo "Updated .bashrc."
    fi

    # the @update.afni.binaries doesn't do the .bash_profile
    touch ~/.bash_profile
    temp_grep=`grep -c "apsearch" ~/.bash_profile`
    if [ "$temp_grep" = "0" ]; then
        # echo export 'PATH=$PATH:'${bindir} >> ~/.bash_profile
        echo 'ahdir=`apsearch -afni_help_dir`' >> ~/.bash_profile
        echo 'if [ -f "$ahdir/all_progs.COMP.bash" ]' >> ~/.bash_profile
        echo "then" >> ~/.bash_profile
        echo '   . $ahdir/all_progs.COMP.bash' >> ~/.bash_profile
        echo "fi" >> ~/.bash_profile
        echo "Updated .bash_profile."
    fi
    temp_grep=`grep -c "/opt/X11/lib/flat_namespace" ~/.bash_profile`
    if [ "$temp_grep" = "0" ]; then
        echo ${bash_add} >> ~/.bash_profile
    fi

    ## afnirc etc
    if [ ! -f "$HOME/.afnirc" ]; then
        \cp ${bindir}/AFNI.afnirc $HOME/.afnirc
        echo "Updated .afnirc."
    fi
    if [ ! -f "$HOME/.sumarc" ]; then
        suma -update_env
        echo "Updated suma environment."
    fi
    echo ""
    echo "Done updating .cshrc, .bashrc, .bash_profile, and environment"
    echo ""
fi   ## end AFNI install check

###########################################################################
## class data
if [[ "${inst_list[@]}" =~ "Class" ]]; then

    ## check for afni
    has_afni

    ## ask for default folder
    text1="Do you want to install AFNI class data in the default location"
    text2="(~/) or somewhere else?" ; text=`echo ${text1} ${text2}`
    title="Where should we put the class data?"
    choice1="~/" ; choice2="Elsewhere"
    data_dest=`AppleChoose2`

    ## get target folder
    if [ $data_dest == "Elsewhere" ]; then
        text="Select or create target a folder for class data." ; folder="$HOME"
        data_dir=`AppleFolder`
        if [ "$data_dir" = "" ]; then
            text1="You cancelled folder selection!\n"
            text2="Please try again if you make up your mind."
            text=`echo ${text1}${text2}`
            title="Folder selection cancelled!" ; button="Quit!" ; icon=0
            junk=`AppleInfo` ; CleanBad
        fi
    else
        data_dir="$HOME"
    fi

    ## give warning
    text1="The class data download may take a very long time."
    text2="You can watch the progress in the terminal."
    text=`echo ${text1} ${text2}`
    title="Prepare to wait..." ; button="GO!" ; icon=2 ; junk=`AppleInfo`

    ## download data
    echo ""
    echo "******************************************************"
    echo "Downloading AFNI class data." ; echo ""
    curl -LO https://afni.nimh.nih.gov/pub/dist/edu/data/CD.tgz

    ## check and install
    if [ -f "CD.tgz" ]; then
        tar xvzf CD.tgz
        cd CD
        tcsh s2.cp.files . "${data_dir}"
        cd $inst_dir
        \rm -rfv CD
        \rm -fv CD.tgz
    else
        text1="Please try again or follow the instructions here:\n\n"
        text2="https://afni.nimh.nih.gov/pub/dist/doc/htmldoc/background_install/install_instructs/steps_mac_app.html#id9"
        text=`echo ${text1}${text2}`
        title="Class data download failed!" ; button="Quit!" ; icon=0
        junk=`AppleInfo` ; CleanBad
    fi
fi   ## end class data

###########################################################################
## afni_system_check
if [[ "${inst_list[@]}" =~ "afni_system_check" ]]; then

    ## check for afni
    has_afni

    echo "******************************************************"
    echo "Running afni_system_check.py -check_all" ; echo ""
    ## save out with time stamp
    cur_time=`date "+%Y%m%d-%H%M%S"`
    afni_system_check.py -check_all > ${HOME}/Desktop/afni_system_check.${cur_time}.txt

    ## view or skip
    text1="afni_system_check.${cur_time}.txt is saved to your Desktop."
    text2="Would you like to view it?"
    text=`echo ${text1} ${text2}`
    title="afni_system_check.py completed!"
    choice1="View" ; choice2="Skip"
    open_afni_check=`AppleChoose2`
    if [ "$open_afni_check" = "View" ]; then
        open -a TextEdit ${HOME}/Desktop/afni_system_check.${cur_time}.txt
    fi
fi   ## end afni_system_check


###########################################################################
## shiny install
if [[ "${inst_list[@]}" =~ "Shiny" ]]; then

    ## check for afni
    has_afni
    afni_bin=`which afni`
    afni_path=$(dirname "${afni_bin}")

    echo "******************************************************"
    echo "Running @afni_R_package_install -shiny -circos" ; echo ""

    ## install shiny packages
    tcsh ${afni_path}/@afni_R_package_install -shiny -circos

    ## check to see if it failed
    if [[ $? != 0 ]]; then
        text1="@afni_R_package_install failed!"
        text2="Try at the terminal with:\n\n"
        text3="tcsh @afni_R_package_install -shiny -circos"
        text=`echo ${text1} ${text2}${text3}`
        title="@afni_R_package_install installation failed!"
        button="Quit!" ; icon=0 ; junk=`AppleInfo` ; CleanBad
    fi
fi   ## end shiny

###########################################################################
## homebrew
if [[ "${inst_list[@]}" =~ "Homebrew" ]]; then

    brew_inst="no"
    brew_bin=`which brew`
    if [ "$brew_bin" != "" ]; then
        text="Do you want to reinstall Homebrew?"
        title="Homebrew is already installed!"
        choice1="Reinstall" ; choice2="Continue to package install"
        brew_reinstall=`AppleChoose2`
        if [[ "${brew_reinstall[@]}" =~ "Reinstall" ]]; then
            brew_inst="yes"
        fi
    else
        brew_inst="yes"
    fi   ## end check if homebrew is there

    ############################################
    ## install homebrew
    if [ "$brew_inst" = "yes" ]; then

        ## install and check to see if it failed
        echo "******************************************************"
        echo "Installing Homebrew:" ; echo ""
        brew_cmd
        if [[ $? != 0 ]]; then
            text1="Homebrew installation failed!!\n\n"
            text2="Try again with instructions from here:\n\n"
            text3="https://brew.sh/"
            text=`echo ${text1}${text2}${text3}`
            title="Homebrew installation failed!" ; button="Quit!" ; icon=0
            junk=`AppleInfo` ; CleanBad
        fi
    fi

    ############################################
    ## install packages

    echo ""
    echo "******************************************************"
    echo "Installing homebrew packages" ; echo ""

    ## loop over the packages to see if they are there already
    for pkg in "${brew_pkgs_arr[@]}"
    do
        if brew ls --versions ${pkg} > /dev/null; then
          echo "" ; echo "${pkg} is installed. Skipping." ; echo ""
        else
            ## install and check to see if it failed
            brew install ${pkg}
            if [[ $? != 0 ]]; then
                text1="${pkg} installation failed!!\n\n"
                text2="Try again at the terminal:\n\n"
                text3="brew install ${pkg}"
                text=`echo ${text1}${text2}${text3}`
                title="Homebrew package installation failed!"
                button="Quit!" ; icon=0
                junk=`AppleInfo` ; CleanBad
            fi
        fi
    done   ## done brew install loop
fi   ## end homebrew

# ###########################################################################
# ## clang for R install
# if [[ "${inst_list[@]}" =~ "Clang" ]]; then
#
#
#     curl -LO "https://cloud.r-project.org/bin/macosx/tools/clang-${R_clang_ver}.pkg"
#     #
#     # export PATH=/usr/local/clang6/bin:$PATH
#
# fi   ## end clang etc R

###########################################################################
## clean up
text1="It seems like everything completed!\n"
text2="If not, try again with the manual instructions from here:\n\n"
text3="https://afni.nimh.nih.gov/pub/dist/doc/htmldoc/background_install/install_instructs/steps_mac.html\n\n"
text4="You should restart your computer (or at least log out and back in).\n"
text=`echo ${text1}${text2}${text3}${text4}`
title="Installations Complete!" ; button="Finish" ; icon=1
junk=`AppleInfo`
CleanGood

